/**********************************************************************************
 Persistence of Vision Ray Tracer Scene Description File
 File name   :  .pov
 Version     : 3.7 and above
 Description : see accompagning text file
               
 Date        : September 2018
 Author      : Kenneth Walker
 E-mail      : povray.tools.general
 
 Copyright (C) 2018. All rights reserved.
 This file is licensed under the terms of the CC-LGPL.
 Use and/or modification of this scene is free, except for commercial purposes.
 Commercial distribution is not allowed without written permission from the author.
**********************************************************************************/

// 9/1-16/18   [Norbert Kern's original code from 2006,
//              with modifications by Kenneth Walker]

#version 3.8;  // or 3.71 or 3.8

#ifndef (Standalone)	#declare Standalone = on;	#end

//______________________________________________________________________
// creating the macro___________________________________________________
#macro pos_finder (SURFACE, cam_loc, lookat, cam_ang, screen_x_position, screen_y_position, arrow_scale)

#if(cam_loc.z <= 0)
  // do nothing
#else
  #local screen_x_position = 1000 - screen_x_position;
#end

#local screen_x_position = screen_x_position/10;
#local screen_y_position = screen_y_position/10;

#local cam_mirror = #if(cam_loc.z < lookat.z) -1; #else 1; #end
#local cam_z      = 0.5*image_width/image_height/tan(radians(cam_ang*0.5));
#local cam_a      = cam_mirror*image_width/image_height;
#local cam_s      = y;
#local cam_d      = vnormalize (lookat-cam_loc);
#local cam_r      = vnormalize (vcross(cam_s,cam_d));
#local cam_up     = vnormalize (vcross(cam_d,cam_r));
#local cam_dir    = cam_d*cam_z;
#local cam_right  = cam_r*cam_a;

#local TEXT_OBJECT_TRANSFORM =
transform {
  matrix <
    cam_r.x, cam_r.y, cam_r.z,
    cam_up.x, cam_up.y, cam_up.z,
    cam_d.x, cam_d.y, cam_d.z,
    cam_loc.x, cam_loc.y, cam_loc.z
  >
}

#local dir_y  = vaxis_rotate (lookat-cam_loc, vcross (lookat-cam_loc, cam_up), (0.5-screen_y_position*0.01)*cam_ang/image_width*image_height);
#local dir_xy = vaxis_rotate (dir_y, cam_up, (0.5-screen_x_position*0.01)*cam_mirror*cam_ang);

#local Norm = <0, 0, 0>;
#local Inter = trace (SURFACE, cam_loc, dir_xy, Norm);
#if (vlength (Norm) != 0)
  #local rx = degrees (atan2 (Norm.z, Norm.y + 0.0001));
  #local rz = -degrees (atan2 (Norm.x, Norm.y + 0.0001));
  #local precise = 3;

  union {
    cylinder {0.2*y, 0.6*y, 0.02}
    cone {0, 0, 0.2*y, 0.06}
    text {ttf "cyrvetic.ttf" "POV-Ray" 0.2, 0 scale 0.2 translate <-0.40, 0.5, -0.02>} // "POV-Ray"
    scale arrow_scale*0.0040*vlength(cam_loc - Inter)*cam_ang
    rotate <rx, 0, rz>
    translate Inter
    texture {
      pigment {color rgb 0.6*x}
      finish {emission 0.15 diffuse 0.85 specular 0.3 roughness 0.1}
    }
  }
  union {
    text {
      ttf "cyrvetic.ttf"
        concat ("rotate <", str (rx, 0, precise-1), ", 0, ", str (rz, 0, precise-1),">")
        0.02, 0
      translate 0.6*y
    }
    text {
      ttf "cyrvetic.ttf"
        concat ("translate <", str (Inter.x, 0, precise), ", ", str (Inter.y, 0, precise), ", ", str (Inter.z, 0, precise),">")
        0.02, 0
      translate -0.6*y
    }
    no_shadow
    scale 0.025*(cam_ang)/90
    translate <-0.38*(cam_ang/90), 0.32*(image_height/image_width)*(cam_ang/90), 0.5>
    transform {TEXT_OBJECT_TRANSFORM}
    pigment {color rgb z}
    finish {emission 0.15 diffuse 0.85}
  }

  #debug "Position Finder results:\n"
  #debug concat ("  rotate  <", str (rx, 0, precise-1), ", 0, ", str (rz, 0, precise-1), ">\n")
  #debug concat ("  translate <", str (Inter.x, 0, precise),", ", str (Inter.y, 0, precise), ", ",str (Inter.z, 0, precise), ">\n\n")
#else
  #debug "Position Finder results:\n"
  #debug concat ("  Chosen position not on traced SURFACE, please try other screen_x_position or screen_y_position values\n\n")
#end

camera {
  perspective
  location cam_loc
  look_at lookat
  right cam_right
  angle cam_ang
  direction -z
}

#end  // [end of macro]
//______________________________________________________________________

#if (Standalone)

// definitions (for test scene below) _____________________________________
#declare screen_x_position = 806; // screen percentages X 1000  (0 to 1000)
#declare screen_y_position = 326; // ditto

#declare cam_loc = <2,4,-8>;
#declare lookat  = <.3,-.1,-.2>;
#declare cam_ang = 50; // 36

// test scene __________________________________________________________________
global_settings{assumed_gamma 1.0}

background {color rgb .7}

camera {
  perspective
  location cam_loc
  look_at lookat
  right x*image_width/image_height
  angle cam_ang
  //direction -z
}

light_source {0 color rgb .7 translate <-200,200,-200>}
light_source {0 color rgb .7 translate <200,200,200>}

// [optional origin markers; pointing only in the POSITIVE directions]
union{
cylinder{0,.8*x .03 pigment{rgb 1.5}}     // X-- WHITE
cylinder{0,.8*y .03 pigment{rgb <0,1,0>}} // Y-- GREEN
cylinder{0,.8*z .03 pigment{rgb <0,.2,1>}} // Z-- BLUE
text{ttf "cyrvetic.ttf" "+ X" 0.2, 0 scale .28
     translate <.9,.04,0> pigment{rgb 1.5} no_shadow} // " + X"
text{ttf "cyrvetic.ttf" "+ Y" 0.2, 0 scale .28
     translate <-.2,.9,0> pigment{rgb <0,1,0>} no_shadow} // " + Y"
text{ttf "cyrvetic.ttf" "+ Z" 0.2, 0 scale .28
     translate <-.13,.04,.9> pigment{rgb <0,.2,1>} no_shadow} // "+ Z"
      scale 1.3
      translate .19*y
    }

// [The NEW object you want to place in the scene...]
#declare NEW_OBJECT =
union{
cylinder{0,.6*x .06 pigment {rgb 1.5}} //     WHITE for x-axis
cylinder{0,.6*y .06 pigment {rgb <0,1,0>}} // GREEN for y-axis
cylinder{0,.6*z .06 pigment{rgb <0,.2,1>}} //  BLUE for z-axis
box{0,.3 translate -.15 pigment{rgb <1,.7,0>}} // YELLOW
    scale .7
    translate .15*y
}

#declare SURFACE =
union {
  height_field {
    function 500,500 {pattern {bozo scale 0.08}}
    smooth
    translate <-0.5, 0, -0.5>
    scale <7, 0.4, 7>
    texture {
      pigment {
        average
        pigment_map {
          [1.0 rgb <0.5, 1, 0.3>]
          [0.6 gradient x frequency 8
               color_map {
                 [0.07 rgb -0.3]
                 [0.07 rgb <0.7, 1, 0.7>]
               }
             ]
          [0.6 gradient z frequency 8
               color_map {
                 [0.07 rgb -0.3]
                 [0.07 rgb <0.7, 1, 0.7>]
               }
             ]
        }
      }
      finish {ambient 0 diffuse 0.5}
    }
  }

  sphere {0, 0.9
    pigment {
      average
      pigment_map {
        [1.0 planar
             pigment_map {
               [0.5 bumps scale 2]
               [0.5 rgb -0.5]
             }
           ]
        [1.0 planar
             pigment_map {
               [0.5 bumps scale 2]
               [0.5 rgb -0.5]
             }
             rotate 90*z
           ]
        [1.0 planar
             pigment_map {
               [0.5 bumps scale 2]
               [0.5 rgb -0.5]
             }
             rotate 90*x
           ]
      }
      scale 0.03
    }
    translate <-1.8,.85,1.8>
  }

  box {0, 2 translate -1 rotate 10*y rotate 45*x rotate 45*z
    pigment {bumps scale 0.05}
    translate <-1.8, -0.6, -1.5>
  }

  union {
    cylinder {-1.4*z, 1.4*z, 0.7
      pigment {
        average
        pigment_map {
          [1.0 planar
               pigment_map {
                 [0.5 bumps scale 2]
                 [0.5 rgb -0.1]
               }
             ]
          [1.0 planar
               pigment_map {
                 [0.5 bumps scale 2]
                 [0.5 rgb -0.1]
               }
               rotate 90*z
             ]
        }
        scale 0.03
      }
      translate <1.5, 0, 0>
    }

    cylinder {-1.4*z, 1.4*z, 0.7
      pigment {
        average
        pigment_map {
          [1.0 planar
               pigment_map {
                 [0.5 bumps scale 2]
                 [0.5 rgb -0.1]
               }
             ]
          [1.0 planar
               pigment_map {
                 [0.5 bumps scale 2]
                 [0.5 rgb -0.1]
               }
               rotate 90*z
             ]
        }
        scale 0.03
      }
      rotate -90*y
      translate <2.8, 0, 1.4>
    }
    rotate 90*y
    rotate -5*x
    translate <1.5, 0.7, 4.0>
  }
} // end of 'SURFACE' union


object {SURFACE}

// The macro call
pos_finder (SURFACE, cam_loc, lookat, cam_ang, screen_x_position, screen_y_position, 1.0)

// Positioning the new object
object{NEW_OBJECT

// rotate ...*y // [optional, in case you don't like the basic x or
// z 'direction' that the new object is pointing in. This needs to come
// BEFORE the following transforms that are returned from the code]

// [the RETURNED on-screen rotation and translation-- replace with the
// new ones, in this order]
rotate    <5.01,0,5.12>
translate <2.844,0.229,-2.911>
}

#end  //of Standalone


